#include <QSettings>
#include "configuration.h"

// **************************************************************
// Loads program settings from registry or file.
// **************************************************************
void LoadProgramSettings(MainWindow* MainWin)
{
   // Create a QSettings object and read program settings.
   QSettings settings;

   // Load settings for the event list.
   settings.beginGroup("EventList");

   MainWin->Config.EvtListColStat[0] = settings.value("ShowRowNumbers", true).toBool();
   MainWin->Config.EvtListColStat[1] = settings.value("ShowAction", true).toBool();
   MainWin->Config.EvtListColStat[2] = settings.value("ShowModel", true).toBool();
   MainWin->Config.EvtListColStat[3] = settings.value("ShowAgency", true).toBool();
   MainWin->Config.EvtListColStat[4] = settings.value("ShowRMS", true).toBool();
   MainWin->Config.EvtListColStat[5] = settings.value("ShowGap", true).toBool();
   MainWin->Config.EvtListColStat[6] = settings.value("ShowErrLat", true).toBool();
   MainWin->Config.EvtListColStat[7] = settings.value("ShowErrLon", true).toBool();
   MainWin->Config.EvtListColStat[8] = settings.value("ShowErrDep", true).toBool();
   MainWin->Config.EvtListColStat[9] = settings.value("ShowDist", true).toBool();
   MainWin->Config.EvtListColStat[10] = settings.value("ShowType", true).toBool();
   MainWin->Config.EvtListColStat[11] = settings.value("ShowNrSta", true).toBool();
   MainWin->Config.EvtListColStat[12] = settings.value("ShowMW", true).toBool();
   MainWin->Config.EvtListColStat[13] = settings.value("ShowML", true).toBool();
   MainWin->Config.EvtListColStat[14] = settings.value("ShowMC", false).toBool();
   MainWin->Config.EvtListColStat[15] = settings.value("ShowMb", false).toBool();
   MainWin->Config.EvtListColStat[16] = settings.value("Show_MB", false).toBool();
   MainWin->Config.EvtListColStat[17] = settings.value("ShowMs", false).toBool();
   MainWin->Config.EvtListColStat[18] = settings.value("Show_MS", false).toBool();
   MainWin->Config.EvtListColStat[19] = settings.value("ShowLocality", false).toBool();
   MainWin->Config.EvtListColStat[20] = settings.value("ShowSfile", false).toBool();
   settings.endGroup();

   // Load general configuration settings.
   settings.beginGroup("Configuration");
   MainWin->Config.TimeInterval = settings.value("TimeInterval", 31).toInt();
   MainWin->Config.AutoLoad = settings.value("AutoLoad", false).toBool();
   MainWin->Config.LastUsedDatabaseType = settings.value("LastDbType", no_database).toInt();
   MainWin->Config.LastUsedDatabaseName = settings.value("LastDbName").toString();
   MainWin->Config.LastUsedDatabasePath = settings.value("LastDbPath").toString();
   MainWin->Config.UseAltWorkDir = settings.value("UseAlternativeWorkDir", false).toBool();
   MainWin->Config.AltWorkDir = settings.value("AlternativeWorkDir").toString();
   MainWin->Operator.SetUnconfirmedId(settings.value("OperatorId").toString());
   settings.endGroup();

   // Restore the current state of mainwindow's toolbars and dockwidgets.
   restoreState(settings.value("AppState", QByteArray()).toByteArray());
   restoreGeometry(settings.value("AppGeometry", QByteArray()).toByteArray());
}




// ************************************************************
// Saves program settings to registry or file.
// ************************************************************
void SaveProgramSettings(MainWindow* parent)
{
   // Create a QSettings object and save program settings.
   QSettings settings;

   // Save settings for the event list.
   settings.beginGroup("EventList");
   settings.setValue(QString("ShowRowNumbers"), Config.EvtListColStat[0]);
   settings.setValue(QString("ShowAction"), Config.EvtListColStat[1]);
   settings.setValue(QString("ShowModel"), Config.EvtListColStat[2]);
   settings.setValue(QString("ShowAgency"), Config.EvtListColStat[3]);
   settings.setValue(QString("ShowRMS"), Config.EvtListColStat[4]);
   settings.setValue(QString("ShowGap"), Config.EvtListColStat[5]);
   settings.setValue(QString("ShowErrLat"), Config.EvtListColStat[6]);
   settings.setValue(QString("ShowErrLon"), Config.EvtListColStat[7]);
   settings.setValue(QString("ShowErrDep"), Config.EvtListColStat[8]);
   settings.setValue(QString("ShowDist"), Config.EvtListColStat[9]);
   settings.setValue(QString("ShowType"), Config.EvtListColStat[10]);
   settings.setValue(QString("ShowNrSta"), Config.EvtListColStat[11]);
   settings.setValue(QString("ShowMW"), Config.EvtListColStat[12]);
   settings.setValue(QString("ShowML"), Config.EvtListColStat[13]);
   settings.setValue(QString("ShowMC"), Config.EvtListColStat[14]);
   settings.setValue(QString("ShowMb"), Config.EvtListColStat[15]);
   settings.setValue(QString("Show_MB"), Config.EvtListColStat[16]);
   settings.setValue(QString("ShowMs"), Config.EvtListColStat[17]);
   settings.setValue(QString("Show_MS"), Config.EvtListColStat[18]);
   settings.setValue(QString("ShowLocality"), Config.EvtListColStat[19]);
   settings.setValue(QString("ShowSfile"), Config.EvtListColStat[20]);
   settings.endGroup();

   // Save general configuration settings.
   settings.beginGroup("Configuration");
   settings.setValue(QString("TimeInterval"), Config.TimeInterval);
   settings.setValue(QString("AutoLoad"), Config.AutoLoad);
   settings.setValue(QString("LastDbType"), Config.LastUsedDatabaseType);
   settings.setValue(QString("LastDbName"), Config.LastUsedDatabaseName);
   settings.setValue(QString("LastDbPath"), Config.LastUsedDatabasePath);
   settings.setValue(QString("UseAlternativeWorkDir"), Config.UseAltWorkDir);
   settings.setValue(QString("AlternativeWorkDir"), Config.AltWorkDir);
   settings.setValue(QString("OperatorId"), Operator.GetCurrentId());
   settings.endGroup();

   // Save the current state of mainwindow's toolbars and dockwidgets.
   settings.setValue(QString("AppState"), saveState());
   settings.setValue(QString("AppGeometry"), saveGeometry());
}

