#include <QDir>
#include <QString>
#include <QFileDialog>
#include <QMessageBox>
#include "logmodel.h"
#include "configuredlg.h"
#include "ui_configuredlg.h"

ConfigureDlg::ConfigureDlg(QWidget *parent) : QDialog(parent), ui(new Ui::ConfigureDlg)
{
    ui->setupUi(this);
}

ConfigureDlg::~ConfigureDlg()
{
    delete ui;
}


//
// Initialize tyhe dialog box.
//
void ConfigureDlg::InitDialog(app_config *Config, int AppStatus)
{
   // Save a pointer to configuration structure,
   // and the contents of the config structure.
   pConfig = Config; DlgConfig = *Config;

   // Initialize all values in the dialog box.
   ui->spinBox1->setValue(DlgConfig.TimeInterval);
   ui->OpenLast->setChecked(DlgConfig.AutoLoad);
   ui->UseAltWD->setChecked(DlgConfig.UseAltWorkDir);
   ui->WorkDir->setText(DlgConfig.AltWorkDir);
   ui->WorkDir->setEnabled(DlgConfig.UseAltWorkDir);
   if (AppStatus != db_closed) {
      ui->UseAltWD->setDisabled(true);
      ui->WorkDir->setDisabled(true);
      ui->BrowseWorkDirButton->setDisabled(true);
   }
}



//
// User has pressed the 'UseAltWD' check box.
//
void ConfigureDlg::UseAltWD_Clicked()
{
   // Enable or disable the 'WorkDir' box.
   if (ui->UseAltWD->isChecked()) {
      ui->WorkDir->setEnabled(true);
      ui->BrowseWorkDirButton->setEnabled(true);
   } else {
      ui->WorkDir->setEnabled(false);
      ui->BrowseWorkDirButton->setEnabled(false);
   }
}



//
// User has pressed the 'Browse' for work directory button.
//
void ConfigureDlg::BrowseWorkDir_Clicked()
{
   // Bring up folder selection dialog box. Return if user cancels the operation.
   QString Message = "Please select a work directory";
   QString WorkDir = QFileDialog::getExistingDirectory(this, Message, QDir::currentPath(), QFileDialog::ShowDirsOnly);

   // Return if user did not select a folder.
   if (WorkDir.isNull()) return;

   // Use native separators in the path.
   WorkDir = QDir::toNativeSeparators(WorkDir);

   // Update the work directory.
   ui->WorkDir->setText(WorkDir);
}



//
// User has pressed the 'OK' button.
//
void ConfigureDlg::accept(void)
{
   QString Message;

   // Get user spesified working directory.
   DlgConfig.UseAltWorkDir = false;
   if (ui->UseAltWD->isChecked()) {
      // Check that directory exists.
      if (QDir(ui->WorkDir->text()).exists()) {
         // Save directory configuration.
         DlgConfig.UseAltWorkDir = true;
         DlgConfig.AltWorkDir = ui->WorkDir->text();
      } else {
         // Invalid directory entered.
         Message = "Non-exisitng or invalid working directory entered.";
         QMessageBox::critical(NULL, "Seisan Explorer", Message);
         ui->WorkDir->selectAll(); ui->WorkDir->setFocus();
         return;
      }
   }

   // Get other values from the dialog box.
   DlgConfig.TimeInterval = ui->spinBox1->value();
   DlgConfig.AutoLoad = ui->OpenLast->isChecked();

   // Return configuration back to parent class.
   *pConfig = DlgConfig;

   // Close the dialog and return 'Accepted' signal.
   QDialog::accept();
}
